<?php

declare(strict_types=1);

namespace mirzaev\vk\arangodb\traits\HTTP\headers\content;

/**
 * Обработка заголовка: "Content-Disposition"
 *
 * @see https://developer.mozilla.org/ru/docs/Web/HTTP/Headers/Content-Disposition
 */
trait disposition
{
    /**
     * Поиск значения filename
     *
     * @param string $raw Необработанное значение заголовка
     */
    private static function filename(string $raw): ?string
    {
        // Инициализация
        $raw = trim($raw);

        if (strpos($raw, ';') === false) {
            // Разделитель параметров не найден (пустая строка или отсутствие параметров)

            return null;
        }

        // Инициализация
        $params = [];
        $_params = explode(';', $raw);
        $type = $_params[0];
        unset($_params[0]);

        if (str_contains($type, 'attachment')) {
            // В заголовке указано, что это скачиваемый файл

            foreach ($_params as $param) {
                if (str_contains($param, '=')) {
                    // Параметр подходит по маске: "текст=текст"

                    // Чтение и запись в переменные
                    [$key, $value] = explode('=', $param, 2);

                    // Запись в буфер
                    $params[trim($key)] = trim($value);
                }
            }

            // Инициализация (порядок связан с приоритетом)
            $masks = ['filename*' => true, 'filename' => false];
            $targets = [];

            foreach ($masks as $mask => $utf8) {
                // Перебор значений масок

                if (isset($params[$mask])) {
                    // Параметр из буфера прошел проверку по маске

                    // Инициализация
                    $target = trim($params[$mask]);

                    // Надо разобраться с этим
                    if ($utf8 && strpos($target, "utf-8''") === 0 && $target = substr($target, strlen("utf-8''")))
                        $target = rawurldecode($target);

                    // Надо разобраться с этим
                    if (substr($target, 0, 1) === '"' && substr($target, -1, 1) === '"')
                        $target = substr($target, 1, -1);

                    $targets[$mask] = $target;
                }
            }

            // Возврат по приоритету
            return $targets[$masks[0]] ?? $targets[$masks[1]];
        }

        return null;
    }
}
